var VSSDsTab = AVTreeTab.extend({

   initialize: function() {
      var nCritical = 0;
      var nWarn = 0;
      var nInfo = 0;

      this.model.set({
         'nCritical':nCritical,
         'nWarn':nWarn,
         'nInfo':nInfo
      });
      AVTreeTab.prototype.initialize.call(this);
   },

   drawAll: function() {
      var sDeviceID = this.model.get('sDeviceID');
      var oBodyDiv = this.$('.opBodyInner');
      this.$('.leftColumn').remove();
      var oColumnDiv = $('<div/>', {'class':'leftColumn'});
      oBodyDiv.append(oColumnDiv);
      
      if (sDeviceID == '') {
         this.drawOverviewHeader();
         this.drawSummaryWidget(oColumnDiv);
         this.$('.opFooter').remove();
         TriggerResize();
      }
      else {
         this.drawHeader();
         this.drawDetailsWidget(oColumnDiv, "STR_PHYSICAL_DRIVE_DETAILS");
         this.drawFooter();
         this.drawButtons();
      }
   },

   drawOverviewHeader: function() {
      var oHeaderDiv = this.$('.opHeader').empty();
      var oDeviceLabelDiv = $('<div/>', {'class': 'deviceLabel'});
      oHeaderDiv.append(oDeviceLabelDiv);
      var oMDeviceLabel = new MDeviceLabel({
         'sDeviceID': '',
         'sLabel': TranslateString("STR_WEAR_GAUGE_SUMMARY"),
         'sIconClass': "ICON_SSD_WEAR_GAUGE"
      });
      var oVDeviceLabel = new VDeviceLabel({
         'model': oMDeviceLabel,
         'el': oDeviceLabelDiv
      });
      oHeaderDiv.append(GetClearFloatsDiv());
   },

   drawButtons: function() {
      var oButtonsDiv = this.$('.buttons');
      oButtonsDiv.empty();
      var sLabel = 'STR_BACK_TO_OVERVIEW';
      var sClass = "button popupTabbable pianoButton default BackToOverview";
      var oButtonDiv = $('<a/>', {
         'href':'',
         'id':sLabel,
         'class':sClass
      });
      oButtonDiv.html(TranslateString(sLabel));
      oButtonsDiv.append(oButtonDiv);
   },

   drawSummaryWidget: function(oParentDiv) {
      var oWidgetDiv = this.drawColumnWidget(oParentDiv, "detailsWidget");
      var oBox = this.drawBox(oWidgetDiv);

      var oSSDWearoutList = [];
      var oSSD56DaysList = [];
      var oSSD5PercentList = [];
      var oSSD2PercentList = [];
      var oSSDNotSupported = [];
      var oDevices = this.model.get('oDevices');
      var nTotalSSDs = oDevices.length;
      var nTotalSmartArraySSDs = 0;
      var nTotalNonSmartArraySSDs = 0;
      var nTotalSASSSDs = 0;
      var nTotalSATASSDs = 0;
      var nTotalNVMeSSDs = 0;
      
      for (var i=0; i<nTotalSSDs; ++i)
      {
         var sDeviceID = oDevices[i];
         var oDetails = GetDeviceSection(sDeviceID, 'STR_PHYSICAL_DRIVE_DETAILS');
         var oIDSection = GetDeviceSection(sDeviceID, "L0");
         var sStatus = GetSectionAttribute(oDetails, 'STR_SSD_STATUS_MESSAGE');
         var sInterface = oIDSection.DEVICE_INTERFACE;
         var bNonSA = IsDefined(oIDSection.IS_NONSA) && oIDSection.IS_NONSA == "YES";

         if (sStatus == 'STR_MSG_SSD_SMART_TRIP')
         {
            oSSDWearoutList.push(sDeviceID);
         }
         else if (sStatus == 'STR_MSG_SSD_56DAY_WARNING')
         {
            oSSD56DaysList.push(sDeviceID);
         }
         else if (sStatus == 'STR_MSG_SSD_56_DAY_5PERCENT_LIFE_REMAINING')
         {
            oSSD56DaysList.push(sDeviceID);
            oSSD5PercentList.push(sDeviceID);
         }
         else if (sStatus == 'STR_MSG_SSD_56_DAY_2PERCENT_LIFE_REMAINING')
         {
            oSSD56DaysList.push(sDeviceID);
            oSSD2PercentList.push(sDeviceID);
         }
         else if (sStatus == 'STR_MSG_SSD_5PERCENT_LIFE_REMAINING')
         {
            oSSD5PercentList.push(sDeviceID);
         }
         else if (sStatus == 'STR_MSG_SSD_2PERCENT_LIFE_REMAINING')
         {
            oSSD2PercentList.push(sDeviceID);
         }
         else if (sStatus == 'ATTR_VALUE_STATUS_NOT_SUPPORTED')
         {
            oSSDNotSupported.push(sDeviceID);
         }

         if (bNonSA)
         {
            nTotalNonSmartArraySSDs++;
         }
         else
         {
            nTotalSmartArraySSDs++;
         }

         if (sInterface == "ATTR_VALUE_INTERFACE_SAS_SSD")
         {
            nTotalSASSSDs++;
         }
         else if (sInterface == "ATTR_VALUE_INTERFACE_SATA_SSD")
         {
            nTotalSATASSDs++;
         }
         else if (sInterface == "ATTR_VALUE_INTERFACE_NVME")
         {
            nTotalNVMeSSDs++;
         }
      }

      var nTotalWearouts = oSSDWearoutList.length;
      var bHasWearout = nTotalWearouts > 0;
      var bHas56Days = oSSD56DaysList.length > 0;
      var bHas2Percent = oSSD2PercentList.length > 0;
      var bHas5Percent = oSSD5PercentList.length > 0;

      var oDeviceDetailsDiv = $('<div/>', {'class': 'deviceDetails'})
      var oDetailsDiv = $('<div/>', {'class': 'detailsDiv'});

      var oSectionsUL = $('<ul/>', {'class': 'sections'});
      var sSectionName = "STR_SUMMARY";
   
      var oSectionLI = $('<li/>', {'class': 'section'});
      var oSectionNameDiv = $('<div/>', {'class': 'sectionName'});
      oSectionNameDiv.html(TranslateString(sSectionName));
      oSectionLI.append(oSectionNameDiv);
      var oSectionDetailsTable = $('<table/>', {'class': 'sectionDetails'});

      // Total SSDs with Wearout
      this.drawStringPair(oSectionDetailsTable, "STR_TOTAL_WEAROUTS", nTotalWearouts);

      // Total SA SSDs
      this.drawStringPair(oSectionDetailsTable, "STR_TOTAL_SA_SSDS", nTotalSmartArraySSDs);

      // Total Non-SA SSDs
      this.drawStringPair(oSectionDetailsTable, "STR_TOTAL_NON_SA_SSDS", nTotalNonSmartArraySSDs);

      // Total SAS SSDs
      this.drawStringPair(oSectionDetailsTable, "STR_TOTAL_SAS_SSDS", nTotalSASSSDs);

      // Total SATA SSDs
      this.drawStringPair(oSectionDetailsTable, "STR_TOTAL_SATA_SSDS", nTotalSATASSDs);

      // Total NVMe SSDs
      this.drawStringPair(oSectionDetailsTable, "STR_TOTAL_NVME_SSDS", nTotalNVMeSSDs);

      // Total SSDs
      this.drawStringPair(oSectionDetailsTable, "STR_TOTAL_SSDS", nTotalSSDs);
      
      oSectionLI.append(oSectionDetailsTable);
      oSectionsUL.append(oSectionLI);

      oDetailsDiv.append(oSectionsUL);
      oDeviceDetailsDiv.append(oDetailsDiv);
      oBox.append(oDeviceDetailsDiv);

      // List SSDs with wearout if any exists.
      if (bHasWearout)
      {
         this.drawStatusTable(
            oSectionsUL,
            "criticalActive",
            "STR_SSD_WITH_SMART_TRIP_WEAROUT",
            "STR_MSG_SSDS_SMART_TRIP",
            oSSDWearoutList);
      }

      // Table: SSDs with 56 days warning.
      if (bHas56Days)
      {
         this.drawStatusTable(
            oSectionsUL,
            "infoActive",
            "STR_SSD_WITH_56DAY_WARNING",
            "STR_MSG_SSDS_56DAY_WARNING",
            oSSD56DaysList);
      }

      // Table: SSDs with 2% usage remaining warning.
      if (bHas2Percent)
      {
         this.drawStatusTable(
            oSectionsUL,
            "infoActive",
            "STR_SSD_WITH_2PERCENT_WARNING",
            "STR_MSG_SSDS_2PERCENT_WARNING",
            oSSD2PercentList);
      }

      // Table: SSDs with 5% usage remaining warning.
      if (bHas5Percent)
      {
         this.drawStatusTable(
            oSectionsUL,
            "infoActive",
            "STR_SSD_WITH_5PERCENT_WARNING",
            "STR_MSG_SSDS_5PERCENT_WARNING",
            oSSD5PercentList);
      }
   },

   drawStatusTable: function(oParentDiv, sIconClass, sSectionName, sNote, oDevices) {
      var oSectionLI = $('<li/>', {'class': 'section'});
      var oSectionNameDiv = $('<div/>', {'class': 'sectionName'});
      oSectionNameDiv.html(TranslateString(sSectionName));
      oSectionLI.append(oSectionNameDiv);
      var oSectionDetailsTable = $('<table/>', {'class': 'sectionDetails'});
      // Draw the note
      var sLabel = TranslateString(sNote);
      var oTR = $('<tr/>', {'class': ''});
      var oLabelTD = $('<td/>', {'class': 'label'});
      oLabelTD.html(TranslateString(sLabel));
      oTR.append(oLabelTD);
      oSectionDetailsTable.append(oTR);
      var oDeviceParentIDs = this.model.get('oDeviceParentIDs');
      // Draw each "interesting" item
      for (var i=0; i<oDevices.length; ++i)
      {
         var sDeviceID = oDevices[i];
         var oDevice = GetDeviceIDSection(sDeviceID);
         var oParent = GetDeviceIDSection(oDeviceParentIDs[sDeviceID]);
         var sLabel;
         var sLabelStr;
         var sLabelMiddle = "|" + oDevice.SIZE + "|" +
                                  oDevice.DEVICE_INTERFACE + "|" +
                                  oParent.NAME;
         var sLabelEnd = '';
         if (IsDefinedAndSet(oDevice.LOCATION))
         {
            sLabelStr = "STR_SSD_WEAR_GAUGE_LABEL";
            sLabelEnd = "|" + TranslateString(oDevice.LOCATION);
         }
         else
         {
            sLabelStr = "STR_SSD_WEAR_GAUGE_LABEL_NO_LOCATION";
         }
         sLabel = TranslateString(sLabelStr + sLabelMiddle + sLabelEnd);
         var oTR = $('<tr/>', {'class': ''});
         var oLabelTD = $('<td/>', {
            'class': 'data SSDLink',
            'id': sDeviceID//oDevices[i].oDevice.id
         });
         oLabelTD.html(TranslateString(sLabel));
         oTR.append(oLabelTD);
         oSectionDetailsTable.append(oTR);
      }
      oSectionLI.append(oSectionDetailsTable);
      oParentDiv.append(oSectionLI);
   },

   clickOverviewButton: function(e) {
      this.model.set({'sDeviceID':''});
      this.drawAll();
   },

   drawDetailSection: function(oSSD, oDevice, oParentDiv, sSectionName, sPhysicalDrive) {
      var oSectionLI = $('<li/>', {'class': 'section'});
      var oSectionNameDiv = $('<div/>', {'class': 'sectionName'});
      oSectionNameDiv.html(TranslateString(sSectionName));
      oSectionLI.append(oSectionNameDiv);
      var oSectionDetailsTable = $('<table/>', {'class': 'sectionDetails'});
      var bDrawSection = true;

      if (sSectionName == "STR_SMARTSSD_STATUS")
      {
         // Gather initial info
         var sSmartSSDLogFull = this.getValue(oDevice.SmartSSDWearGauge, TranslateString("STR_LOG_FULL"));
         var bSmartSSDLogFull = IsDefined(sSmartSSDLogFull) && sSmartSSDLogFull == "TRUE";
         var bValidData = oSSD.bSmartSSDSupported && !bSmartSSDLogFull;

         var sSSDWearStatus = oSSD.sStatusText;
         if (!oSSD.bSmartSSDSupported)
         {
            sSSDWearStatus = "STR_NOT_SUPPORTED";
         }
         else if (bSmartSSDLogFull)
         {
            sSSDWearStatus = "STR_LOG_FULL_INFO";
         }
         this.drawStringPair(oSectionDetailsTable, "STR_SSD_WEAR_STATUS", TranslateString(sSSDWearStatus));
         
         if (bValidData)
         {
            // Get utilization information
            var nUtilization;
            var sUtilization = this.getValue(oDevice.SmartSSDWearGauge, "Utilization");
            var nLifeLeft = oSSD.nUsageRemaining;
            var sDaysRemaining = oSSD.sDaysRemaining;
            var bValidLifeLeft = IsDefined(sUtilization) && sUtilization >= 0 && sUtilization <= 100;
            if (bValidLifeLeft)
            {
               nUtilization = parseFloat(sUtilization);
               nLifeLeft = 100 - nUtilization;
               nLifeLeft = Math.round(nLifeLeft*100)/100;
            }
            var sPowerOnHours = this.getValue(oDevice.SmartSSDWearGauge, "STR_POWER_ON_HOURS");
            this.drawStringPair(oSectionDetailsTable, "STR_POWER_ON_HOURS", sPowerOnHours);
            if (IsDefinedAndSet(sDaysRemaining))
            {
               this.drawStringPair(oSectionDetailsTable, "STR_SSD_DAYS_REMAINING", TranslateString("STR_DAYS|" + sDaysRemaining));
            }
            if (bValidLifeLeft)
            {
               this.drawStringPair(oSectionDetailsTable, "STR_LIFE_REMAINING", TranslateString("STR_PERCENT|" + nLifeLeft));
               this.drawStringPair(oSectionDetailsTable, "ATTR_NAME_EXINFO_SSD_UTILIZATION", TranslateString("STR_PERCENT|" + nUtilization));
            }
         }
      }
      else if (sSectionName == "STR_PHYSICAL_DRIVE")
      {
         var sSerialNumber, sFirmwareRevision;
         if (sPhysicalDrive == "PhysicalDriveInfo")
         {
            oPhysicalDrive = oDevice.PhysicalDriveInfo;
            sSerialNumber = this.getValue(oPhysicalDrive, "Serial Number");
            sFirmwareRevision = this.getValue(oPhysicalDrive, "Firmware Revision");
         }
         else
         {
            oPhysicalDrive = oDevice.PhysicalDriveStatus;
            sSerialNumber = this.getValue(oPhysicalDrive, "Drive Serial Number");
            sFirmwareRevision = this.getValue(oPhysicalDrive, "Drive Firmware Revision");
         }
         // Display Status
         var sPhysicalStatus = oSSD.sPhysicalStatus;
         if (IsDefined(sPhysicalStatus))
         {
            this.drawStringPair(oSectionDetailsTable, "STR_PHYSICAL_DRIVE_STATUS", sPhysicalStatus);
         }
         var sSize = oSSD.nSize;
         if (IsDefined(sSize))
         {
            this.drawStringPair(oSectionDetailsTable, "ATTR_NAME_TOTAL_BLOCKS", sSize);
         }
         var sSSDInterface = oSSD.sInterface;
         if (IsDefined(sSSDInterface))
         {
            this.drawStringPair(oSectionDetailsTable, "ATTR_NAME_LOGICAL_DRIVE_TYPE", sSSDInterface);
         }
         this.drawAttribute(oSectionDetailsTable, oPhysicalDrive, "Drive Model", "ATTR_NAME_DRIVE_MODEL");

         if (IsDefined(sSerialNumber))
         {
            this.drawStringPair(oSectionDetailsTable, "ATTR_NAME_SERIAL_NUMBER", sSerialNumber);
         }
         if (IsDefined(sFirmwareRevision))
         {
            this.drawStringPair(oSectionDetailsTable, "STR_FIRMWARE_REVISION", sFirmwareRevision);
         }
         //this.DisplayTemp("Current Temperature", oPhysicalDrive);
         //this.DisplayTemp("Maximum Temperature", oPhysicalDrive);
         this.drawAttribute(oSectionDetailsTable, oPhysicalDrive, "ATTR_NAME_CURRENT_TEMPERATURE");
         this.drawAttribute(oSectionDetailsTable, oPhysicalDrive, "ATTR_NAME_MAXIMUM_TEMPERATURE");
         //this.drawAttribute(oSectionDetailsTable, oPhysicalDrive, "GUI_NAME_PHY_COUNT");
         //this.drawAttribute(oSectionDetailsTable, oPhysicalDrive, "ATTR_NAME_NEGOTIATED_LINK_RATE", "GUI_NAME_TRANSFER_SPEED");
      }
      else if (sSectionName == "STR_PHYSICAL_DRIVE_AUTHENTICATION")
      {
         if (sPhysicalDrive == "PhysicalDriveInfo")
         {
            oPhysicalDrive = oDevice.PhysicalDriveInfo;
         }
         else
         {
            oPhysicalDrive = oDevice.PhysicalDriveStatus;
         }
         var sDriveState = this.getValue(oPhysicalDrive, "STR_DRIVE_AUTENTICATION");
         var bDriveAuthSupported = IsDefinedAndSet(sDriveState);
         var sCarrierState = this.getValue(oPhysicalDrive, "STR_SMART_CARRIER_AUTHENTICATION");
         var bCarrierAuthSupported = IsDefinedAndSet(sCarrierState);
         if (!bCarrierAuthSupported || sCarrierState == 0)
         {
            sCarrierState = "STR_NOT_APPLICABLE";
            // Drive authentication is not applicable if carrier 
            // authentication is not applicable.
            sDriveState = "STR_NOT_APPLICABLE";
         }
         else
         {
            switch (sCarrierState)
            {
               case 1:
                  sCarrierState = "STR_OK";
                  break;
               case 2:
                  sCarrierState = "STR_AUTHENTICATION_FAILED_COMMUNICATION";
                  break;
               case 255:
               default:
                  sCarrierState = "STR_AUTHENTICATION_FAILED_NOT_HP";
                  break;
            }

            if (!bDriveAuthSupported)
            {
               sDriveState = "STR_NOT_APPLICABLE";
            }
            else
            {
               switch (sDriveState)
               {
                  case 0:
                     sDriveState = "STR_AUTHENTICATION_FAILED_FIRST";
                     break;
                  case 1:
                     sDriveState = "STR_AUTHENTICATION_FAILED_SECOND";
                     break;
                  case 0x80:
                     sDriveState = "STR_OK";
                     break;
                  default:
                     sDriveState = "STR_AUTHENTICATION_FAILED_ONE";
                     break;
               }
            }
         }
         // Only show the drive authentication state if it's applicable.
         if (sDriveState != "Not Applicable")
         {
            this.drawStringPair(oSectionDetailsTable, "ATTR_NAME_CARRIER_AUTHENTICATION_STATUS", TranslateString(sDriveState));
         }
         // If controller supports drive and carrier authentication, always 
         // show the carrier authentication state, even when not applicable.
         this.drawStringPair(oSectionDetailsTable, "STR_DRIVE_CARRIER_AUTHENTICATION_STATUS", TranslateString(sCarrierState));
         // The two version fields will only contain meaningful data if 
         // the carrier has authenticated successfully (it's in OK state).
         if (sCarrierState == "OK")
         {
            this.drawAttribute(oSectionDetailsTable, oPhysicalDrive, "SMART_CARRIER_APPLICATION_VERSION");
            this.drawAttribute(oSectionDetailsTable, oPhysicalDrive, "SMART_CARRIER_BOOTLOADER_VERSION");
         }
      }
      else if (sSectionName == "STR_LOCATION")
      {
         if (sPhysicalDrive == "PhysicalDriveInfo")
         {
            oPhysicalDrive = oDevice.PhysicalDriveInfo;
         }
         else
         {
            oPhysicalDrive = oDevice.PhysicalDriveStatus;
         }
         // Location
         var sAttrBox = this.getValue(oPhysicalDrive, "Physical Box");
         var sAttrBay = this.getValue(oPhysicalDrive, "Physical Bay");
         var sAttrPort = this.getValue(oPhysicalDrive, "Physical Port");
         if (IsDefined(sAttrBox))
         {
            // Convert to int
            sAttrBox = parseInt(sAttrBox);
            sAttrBay = parseInt(sAttrBay);

            // Display Port
            if (IsDefined(sAttrPort))
            {
               this.drawStringPair(oSectionDetailsTable, "ATTR_NAME_PORT", sAttrPort);
            }
            // Display Box
            if (sAttrBox == "0")
            {
               this.drawStringPair(oSectionDetailsTable, "ATTR_NAME_BOX", TranslateString("STR_ZERO_NOT_APPLICABLE"));
            }
            else
            {
               this.drawStringPair(oSectionDetailsTable, "ATTR_NAME_BOX", sAttrBox);
            }
            // Display Bay
            if (IsDefined(sAttrBay))
            {
               if (sAttrBay == "0" && sAttrBox == "0")
               {
                  this.drawStringPair(oSectionDetailsTable, "ATTR_NAME_BAY", TranslateString("STR_ZERO_NOT_APPLICABLE"));
               }
               else
               {
                  this.drawStringPair(oSectionDetailsTable, "ATTR_NAME_BAY", sAttrBay);
               }
            }
            // Display Location Hint
            /*var sLocHint = this.getValue(oPhysicalDrive, "Location Hint");
            if (IsDefined(sLocHint) && sLocHint != "None")
            {
               this.DisplayAttr("Location Hint");
            }*/
         }
         else
         {
            bDrawSection = false;
         }
      }
      if ( bDrawSection )
      {
         oSectionLI.append(oSectionDetailsTable);
         oParentDiv.append(oSectionLI);
      }
   },

   drawStringPair: function(oParentDiv, sLabel, sData) {
      var oTR = $('<tr/>', {'class': ''});
      var oLabelTD = $('<td/>', {'class': 'label'});
      var oDataTD = $('<td/>', {'class': 'data'});
      oLabelTD.html(TranslateString(sLabel));
      oDataTD.html(sData);
      oTR.append(oLabelTD);
      oTR.append(oDataTD);
      oParentDiv.append(oTR);
   },

   getValue: function(o, sAttrName) {
      var sValue;
      var bFound = false;
      if (IsDefined(o))
      {
         for (i=0; i<o.length && !bFound; ++i)
         {
            bFound = o[i].id == TranslateString(sAttrName);
            if (bFound)
            {
               sValue = o[i].value;
            }
         }
      }
      return sValue;
   },

   drawAttribute: function(oParentDiv, oDevice, sAttrName, sDisplayName) {
      var bDrawAttribute = true;
      if (!IsDefined(sDisplayName))
      {
         var sDisplayName = TranslateString(sAttrName);
      }
      else {
         sDisplayName = TranslateString(sDisplayName);
      }
      var sValue = this.getValue(oDevice, sAttrName);
      if (sAttrName == "GUI_NAME_PHY_COUNT")
      {
         sValue = parseInt(sValue, 16);
      }
      else if (sAttrName == "ATTR_NAME_NEGOTIATED_LINK_RATE") 
      {
         sValue = parseInt(sValue, 16);
      }
      else if (sAttrName == "ATTR_NAME_MAXIMUM_TEMPERATURE" ||
               sAttrName == "ATTR_NAME_CURRENT_TEMPERATURE")
      {  
         sValue = parseInt(sValue, 16);
         if ( isNaN(sValue) || sValue == 0 )
         {
            bDrawAttribute = false;
         }
         sValue = sValue + "&deg;C";
      }
      if ( bDrawAttribute )
      {
         this.drawStringPair(oParentDiv, sDisplayName, sValue);
      }
   },

   drawFooter: function() {
      var oOpFooterDiv = $('<div/>', {
         'class':"opFooter"
      });
      var oInfoDiv = $('<div/>', {
         'class':"info"
      });
      var oLoadingDiv = $('<div/>', {
         'class':"loading"
      });
      var oButtonsDiv = $('<div/>', {
         'class':"buttons"
      });
      oOpFooterDiv.append(oInfoDiv);
      oOpFooterDiv.append(oLoadingDiv);
      oOpFooterDiv.append(oButtonsDiv);
      $(this.el).append(oOpFooterDiv);
   },

   drawDetailedStatus: function() {
      this.model.set({'sDeviceID':'status'});
      this.drawAll();
      return false;
   }
   
});