var MDeviceLabel = Backbone.Model.extend({

   defaults: {
      'sLabel': '',
      'sDescription': '',
      'sIconClass': '',
      'sIconActivityClass': '',
      'sIconTypeOverlayClass': '',
      'sIconStatusOverlayClass': '',
      'sIconStateOverlayClass': '',
      'bEnableLinks': false,
      'sLink': ''
   },

   initialize: function() {
      var sDeviceID = this.get("sDeviceID");
      var sLabel = this.get('sLabel');
      if (IsDefinedAndSet(sDeviceID) && !IsDefinedAndSet(sLabel))
      {
         var section = GetDeviceIDSection(sDeviceID);//, true);
         if (IsDefined(section))
         {
            sLabel = section.LABEL;
            //var sName = section.LOCATION;
            var sDescription = '';//section.SIZE;
            var sIconExtension = '';
            var sIcon = section.ICON;
            var sIconLabel = "";
            var sItemLabel = TranslateString(sLabel);
            var sItemTitle = TranslateString(sDescription);
            //var sItemSubtitle = TranslateString(sName);
            var sDeviceType = section.DEVICE_TYPE;
            // If Controller
            if (sDeviceType == "TypeSmartArrayController")
            {
               sItemLabel = TranslateString(section.NAME);
               // Check for HBA mode
               var sControllerMode = "";
               var sHBAModeEnabled = "";
               var sAdapterType = "";
               if (IsDefinedAndSet(section.CONTROLLER_MODE))
               {
                  sControllerMode = section.CONTROLLER_MODE;
               }
               if (IsDefinedAndSet(section.HBA_MODE_ENABLED))
               {
                  sHBAModeEnabled = section.HBA_MODE_ENABLED;
               }
               if (IsDefinedAndSet(section.ADAPTER_TYPE))
               {
                  sAdapterType = section.ADAPTER_TYPE;
               }

               var bHBAMode = false;
               var bIsSmartHBA = false;
               var bIsMixed = false;
               var sItemLabelModeSuffix = "";
               // Make sure we're not in mixed or unknown mode
               if (sControllerMode == "ATTR_VALUE_CONTROLLER_MODE_UNKNOWN")
               {
                  sItemLabelModeSuffix = TranslateString("STR_UNSUPPORTED_MODE_LABEL");
               }
               else
               {
                  if (sControllerMode == "ATTR_VALUE_CONTROLLER_MODE_HBA" ||
                      sHBAModeEnabled == "ATTR_VALUE_HBA_MODE_ENABLED_TRUE")
                  {
                     bHBAMode = true;
                  }
                  if (sAdapterType == "ATTR_VALUE_ADAPTER_TYPE_SMART_HBA")
                  {
                     bIsSmartHBA = true;
                  }
                  if (sControllerMode == "ATTR_VALUE_CONTROLLER_MODE_MIXED")
                  {
                     bIsMixed = true;
                  }
                  if (bIsSmartHBA && !bHBAMode && !bIsMixed)
                  {
                     // Smart HBA is in RAID Mode
                     sItemLabelModeSuffix = TranslateString("STR_RAID_MODE_LABEL");
                  }
                  else if (!bIsSmartHBA && bHBAMode)
                  {
                     // Smart Array in HBA Mode
                     sItemLabelModeSuffix = TranslateString("STR_HBA_MODE_LABEL");
                  }
               }
               sItemLabel += sItemLabelModeSuffix;

               sDescription = TranslateString(section.LOCATION);
            }
            // Non Smart Array Controllers, NVMe Controllers...
            else if (sDeviceType == "TypeHostBusAdapter")
            {
               sItemLabel = TranslateString(section.LABEL);
               // Non Smart Array Controller
               if (sIcon == "ICON_NVME_CONTROLLER")
               {
                  // Nothing for description ATM...
               }
               else
               {
                  // Show Non Smart Array Controller description for HP only.
                  if (isPersonality("HP"))
                  {
                     sDescription = TranslateString("STR_NON_SMART_ARRAY_CONTROLLER");;
                  }
               }
            }
            // If Drive Cage
            else if (sIcon == "ICON_STORAGE_ENCLOSURE_BACKPLANE" ||
                     sIcon == "ICON_STORAGE_ENCLOSURE_EXTERNAL")
            {
               sItemLabel = TranslateString(section.LABEL);
               sDescription = TranslateString(section.LOCATION);
            }
            // If Port
            else if (sIcon == "ICON_PORT")
            {
               sItemLabel = TranslateString(section.LABEL);
               sDescription = TranslateString(section.ATTR_NAME_PORT_MODE);
            }
            // If Array
            else if (sIcon == "ICON_SAS_ARRAY" || sIcon == "ICON_SATA_ARRAY")
            {
               sIconLabel = TranslateString(section.DEVICE_INTERFACE);
               // sItemTitle = TranslateString(sLabel);// + "|" + section.LD_COUNT);
               // sItemSubtitle = TranslateString(section.FREE_SPACE);
               sItemLabel = TranslateString(sLabel);
               sDescription = TranslateString("STR_ARRAY_DESCRIPTION|" + TranslateString(section.FREE_SPACE));
            }
            // If Logical Drive
            else if (sIcon == "ICON_DATA_LOGICAL_DRIVE")
            {
               sIconLabel = TranslateString(section.RAID_LEVEL);
               // sItemTitle = TranslateString(sDescription);
               // sItemSubtitle = TranslateString(sLabel);
               
               var sRAID = TranslateString(section.RAID_LEVEL);
               sItemLabel = TranslateString(sLabel);
               /*sItemLabel += " (" + section.SIZE;
               if (IsDefinedAndSet(sRAID))
               {
                  sItemLabel += ", " + sRAID;
               }
               sItemLabel += ")";*/
               sDescription = section.SIZE;
               if (IsDefinedAndSet(sRAID))
               {
                  sDescription += ", " + sRAID;
               }
            }
            // If Physical Drive
            else if (sIcon == "ICON_SAS_PHYSICAL_DRIVE" ||
                     sIcon == "ICON_SASSSD_PHYSICAL_DRIVE" ||
                     sIcon == "ICON_SATA_PHYSICAL_DRIVE" ||
                     sIcon == "ICON_SATASSD_PHYSICAL_DRIVE" ||
                     sIcon == "ICON_NVME_PHYSICAL_DRIVE")
            {
               sIconLabel = TranslateString(section.DEVICE_INTERFACE);
               // sItemTitle = TranslateString(sDescription);
               // sItemSubtitle = TranslateString(sName);
               //sItemLabel = TranslateString(sLabel);
               sItemLabel = TranslateString('STR_PDRIVE_NAME_LABEL' + '|' + section.SIZE + '|' + section.DEVICE_INTERFACE);
               sDescription = TranslateString(section.LOCATION);
               if (sDescription == "")
               {
                  sDescription = TranslateString(section.SERIAL_NUMBER);
               }
               // Check which physical drive icon we should show:
               // - Non-spare Assigned
               // - Spare Assigned
               var sDriveType = section.DEVICE_DRIVE_TYPE;
               if (sDriveType == "ATTR_VALUE_DRIVE_TYPE_SPARE" ||
                   sDriveType == "ATTR_VALUE_DRIVE_TYPE_SPARE_ROAMING" ||
                   sDriveType == "ATTR_VALUE_DRIVE_TYPE_SPARE_SHARED")
               {
                  sIconExtension = "_STANDBY_SPARE";
                  // Don't show the following label suffix unless a QUIX asks for it.
                  //sItemLabel += " (" + TranslateString(sDriveType) + ")";
               }
               else if (sDriveType == "ATTR_VALUE_DRIVE_TYPE_SPARE_ACTIVE")
               {
                  sIconExtension = "_ACTIVE_SPARE";
               }
               else if (sDriveType == "ATTR_VALUE_DRIVE_TYPE_UNASSIGNED" || sDriveType == "ATTR_VALUE_DRIVE_TYPE_HBA_MODE_PENDING")
               {
                  sIconExtension = "_UNASSIGNED";
               }
            }
            else if (sIcon == "ICON_LICENSED_FEATURE")
            {
               var sKey = section.ATTR_NAME_LICENSE_KEY;
               if (IsDefinedAndSet(sKey))
               {
                  sDescription = TranslateString("STR_LICENSE_KEY_DESCRIPTION|"+sKey);
               }
            }
            else if (sIcon == "ICON_ENCRYPTION_MANAGER")
            {
               sDescription = TranslateString(section.LOGGED_IN);
               if (this.get('bEnableLinks'))
               {
                  var sLoginOperation = section.LOGIN_OPERATION;
                  if (IsDefinedAndSet(sLoginOperation))
                  {
                     this.set({
                        'sLink': sLoginOperation
                     });
                  }
               }
            }

            // Icon Class
            var sIconClass = sIcon + sIconExtension;

            // Icon Inactive Task Overlay Class
            var sIconInactiveTaskClass = section.ICON_OVERLAY_INACTIVE_TASK;

            // Icon Activity Overlay Class
            var sIconActivityClass = section.ICON_OVERLAY_ACTIVITY;

            // Icon Type Overlay Class
            var sIconTypeOverlayClass = section.ICON_OVERLAY_TYPE;

            // Icon Status Overlay Class
            var sIconStatusOverlayClass = section.ICON_OVERLAY_STATUS;

            // Icon State Overlay Class
            var sIconStateOverlayClass = section.ICON_OVERLAY_STATE;

            this.set({
               'sLabel':sItemLabel,
               'sDescription':sDescription,
               'sIconClass':sIconClass,
               'sIconInactiveTaskClass':sIconInactiveTaskClass,
               'sIconActivityClass':sIconActivityClass,
               'sIconTypeOverlayClass':sIconTypeOverlayClass, // Currently not used.
               'sIconStatusOverlayClass':sIconStatusOverlayClass,
               'sIconStateOverlayClass':sIconStateOverlayClass
            });
         }
      }
   }
   
});